-- Naughty __o/ Notification library for awesome3 HEAD
-- | Tue, 14 Oct 2008 14:36:11 +0100
-- / \
-- koniu <gkusnierz@gmail.com>
-- 
--
-- Usage:
--
-- require("naughty")
-- naughty.notify({
--       text = "notification",
--       title = "title",
--       position = "top_left"|"top_right"|"bottom_left"|"bottom_right",
--       timeout = 5,
--      icon="/path/to/image",
--      fg="#ffggcc",
--      bg="#bbggcc",
--      screen = 1,
--      
-- })
--
--
-- Notes/todo:
--
-- * none of the arguments to notify() is mandatory, for defaults see config dictionary below
-- * would be nice to have borders (need border_width/border_color in wibox)
-- * you can run naughty.notify(args) from luaprompt to test it,
-- feeding it real data (dbus/hal/acpi/*logd/??) is a DIFFERENT story
 
 
 
-- package env
 
local print = print
local pairs = pairs
local table = table
local beautiful = beautiful
local wibox = wibox
local image = image
local awful = awful
local string = string
local widget = widget
local button = button
local capi = { screen = screen }
 
local module_name = "naughty"
module("naughty")
 
 
 
-- initialize config with default values
config = {}
config.margin     = 5
config.height     = 16
config.width     = 300
config.screen     = 1
config.timeout    = 5
config.position   = "top_right"
config.font    = "Verdana 8"
config.icon    = nil
config.bg    = beautiful.bg_focus or "white"
config.fg    = beautiful.fg_focus or "black"
config.border_width  = 3
config.border_color  = 'red'
 
 
-- initialize variables
notifications = {
  top_left = {},
  top_right = {},
  bottom_left = {},
  bottom_right = {},
}
 
local ws = capi.screen[config.screen].workarea
local ss = capi.screen[config.screen].coords
 
 
 
--- Evaluate desired position of the notification by index
-- @param idx Index of the notification
-- @param position One of 4 keys in notification dictionary: top_right, top_left, bottom_right, bottom_left
-- @param lines Number of text lines in the notification #FIXME#
-- @return Absolute position in {x, y} dictionary
local function get_offset(idx, position, lines)
  local v = {}
 
  -- calculate x
  if position:match("left") then v.x = ws.x + config.margin
    else v.x = ws.x + ws.width - config.width - config.margin end
 
  -- calculate existing popups' height
  local existing = 0
  for i = 1, idx-1, 1 do
     existing = existing + config.height*notifications[position][i].lines + config.margin end
 
  -- calculate y
  if position:match("top") then v.y = ws.y + config.margin + existing
    else v.y = ws.y + ws.height - config.margin - config.height*lines - existing end
 
  return v
end
 
 
--- Arrange notifications according to their position
local function arrange()
  for p,pos in pairs(notifications) do
    for i,notification in pairs(notifications[p]) do
       notification.box:geometry({ y = get_offset(i, notification.position, notification.lines).y })
      for i,widget in pairs(notification.box:widgets()) do
        widget:buttons({ button({ }, 1, function () destroy(i,p) end), }) end
    end
  end
end
 
 
--- Destroy notification by index
-- @param idx Index of the notification
-- @param position One of 4 keys in notification dictionary: top_right, top_left, bottom_right, bottom_left
-- @return None
function destroy(idx, position)
  notifications[position][idx].box.screen = nil
  awful.hooks.timer.unregister(notifications[position][idx].timer)
  table.remove(notifications[position], idx)
  arrange()
end
 
 
 
--- Create notification
-- @param args Dictionary of arguments FIXME
-- text, timeout, title, position, icon, fg, bg, screen
function notify(args)
 
  -- gather settings together
  local timeout = args.timeout or config.timeout
  local position = args.position or config.position
  local icon = args.icon or config.icon
  local text = args.text or ""
  local screen = args.screen or config.screen
 
  local title = ""
  if args.title then title = " " .. args.title .. "\n"
      else title = "" end
  
  local lines = 1
    for i in string.gmatch(title..text, "\n") do lines = lines + 1 end
 
  -- create the container wibox
  local idx = #notifications[position] + 1
  local box = wibox({
                name = "not" .. idx,
                position = "floating",
                fg = args.fg or config.fg,
                bg = args.bg or config.bg,
    border_color = config.border_color,
    border_width = config.border_width,
        })
 
  -- position the wibox
        local offset = get_offset(idx, position, lines)
        box:geometry({
                width = config.width,
                height = config.height * lines,
                x = offset.x,
                y = offset.y,
        })
 
        box.ontop = true
        box.screen = screen
 
  -- populate the wibox with widgets
        local textbox = widget({ type = "textbox", name = "text", align = "flex" })
        textbox:buttons({ button({ }, 1, function () destroy(idx, position) end) })
        textbox.text = string.format('<span font_desc="%s"><b>%s</b> %s</span>', config.font, title, text)
       
         local iconbox = nil
  if icon then
    iconbox = widget({ type = "imagebox", name = "iconbox", align = "left" })
          iconbox:buttons({ button({ }, 1, function () destroy(idx, position) end) })
    iconbox.image = image(icon)
    iconbox.width = 20
  end
        
  box:widgets({ iconbox, textbox })
  
  local timer = function () destroy(idx, position) end
  awful.hooks.timer.register(timeout, timer)
 
  -- insert the notification to the table
        table.insert(notifications[position], {
    box = box,
    position = position,
    icon = icon,
    text = text,
    lines = lines,
    timer = timer,
  })
end
